using System;

namespace CSharpApplication.ClassMatrixSample
{
    class Matrix
    {
        // 
        int [,] mtr = null;
        // 
        int M = 0, N = 0;

        // ,   
        public int FirstDimension
        {
            get
            {
                return M;
            }
        }
        // ,   
        public int SecondDimension
        {
            get
            {
                return N;
            }
        }
        //    
        public bool IsEmpty
        {
            get
            {
                return mtr == null;
            }
        }
        //   
        public Matrix()
        {
            //    
        }
        //      
        public Matrix(int M, int N)
        {
            if(M <= 0 || N <= 0)
                return;

            this.M = M;
            this.N = N;
            mtr = new int[M, N];
        }
        //   
        public Matrix(Matrix m)
        {
            M = m.M;
            N = m.N;
            //  Clone()  Array 
            //   
            mtr = (int[,])m.mtr.Clone();
        }
        //       
        public Matrix(int [,] m)
        {
            //    
            M = m.GetLength(0);
            N = m.GetLength(1);
            mtr = (int[,])m.Clone();
        }
        //    (  )
        public Matrix Copy()
        {
            //   
            return new Matrix(this);
        }
        //      
        public int this [int i, int j]
        {
            //     
            get
            {
                if(i < 0 || i >= M || j < 0 || j >= N)
                    throw new IndexOutOfRangeException();

                return mtr[i, j];
            }
            //       
            set
            {
                if(i < 0 || i >= M || j < 0 || j >= N)
                    throw new IndexOutOfRangeException();

                mtr[i, j] = value;
            }
        }
        //   +    
        public static Matrix operator + (Matrix m1, Matrix m2)
        {
            //      
            if(m1.M != m2.M || m1.N != m2.N)
                return new Matrix();

            //  

            Matrix Result = new Matrix(m1);
            for(int i = 0; i < Result.M; i++)
            {
                for(int j = 0; j < Result.N; j++)
                {
                    Result.mtr[i, j] += m2.mtr[i, j];
                }
            }

            return Result;
        }
        //   *    
        public static Matrix operator * (Matrix m1, Matrix m2)
        {
            //      
            if(m1.N != m2.M)
                return new Matrix();

            //  

            Matrix Result = new Matrix(m1.M, m2.N);
            for(int i = 0; i < m1.M; i++)
            {
                for(int j = 0; j < m2.N; j++)
                {
                    for(int k = 0; k < m1.N; k++)
                    {
                        Result.mtr[i, j] += m1.mtr[i, k] * m2.mtr[k, j];
                    }
                }               
            }

            return Result;
        }
        //   +     
        public static Matrix operator + (Matrix m1, int value)
        {
            Matrix Result = new Matrix(m1);
            for(int i = 0; i < Result.M; i++)
            {
                for(int j = 0; j < Result.N; j++)
                {
                    Result.mtr[i, j] += value;
                }
            }

            return Result;
        }
        //   *     
        public static Matrix operator * (Matrix m1, int value)
        {
            Matrix Result = new Matrix(m1);
            for(int i = 0; i < m1.M; i++)
            {
                for(int j = 0; j < m1.N; j++)
                {
                    Result.mtr[i, j] *= value;
                }               
            }

            return Result;
        }    
        //     ( )
        public static bool operator != (Matrix m1, Matrix m2)
        {
            if(m1.M != m2.M || m1.N != m2.N)
                return true;

            for(int i = 0; i < m1.M; i++)
            {
                for(int j = 0; j < m1.N; j++)
                {
                    if(m1.mtr[i, j] != m2.mtr[i, j])
                        return true;
                }
            }

            return false;
        }
        //     ( )
        public static bool operator == (Matrix m1, Matrix m2)
        {
            return !(m1 != m2);
        }
        //   (   Object)
        //  Equals    
        public override bool Equals(object obj) 
        {
            try 
            {
                return (bool) (this == (Matrix) obj);
            }
            catch
            {
                //  obj    Matrix
                return false;
            }
        }
        //   (   Object)
        //  GetHashCode   - 
        public override int GetHashCode() 
        {
            //    
            return M ^ N;
        }
        //    
        public void Print()
        {
            string s = new string('-', 79);
            Console.WriteLine(s);
            for(int i = 0; i < M; i++)
            {
                for(int j = 0; j < N; j++)
                {
                    Console.Write("{0,5}", mtr[i, j]);
                }
                Console.WriteLine();
            }
            Console.WriteLine(s);
        }
    }
}