using System;
using System.Drawing;
using System.Windows.Forms;
using System.Diagnostics;
using System.Collections;

//     
namespace CSharpApplication.ProcessViewer
{
    class ProcessViewer : Form
    {
        //  
        ListView ProcessList;
        //   
        Timer RefreshTimer;
        //  
        Button EndProcess;
        //  ,    
        int Column;
        //  
        int SelPos;
        //     
        Label ProcessPath;
        //  
        ToolTip ProcessPathTip;

        static void Main()
        {
            //  
            System.Threading.Mutex m = 
                new System.Threading.Mutex(false, "ProcessViewer.MyMutex");
            //        
            if(m.WaitOne(0, false) == false)
            {
                //  
                MessageBox.Show("This application is already running...",
                    "Warning", MessageBoxButtons.OK, MessageBoxIcon.Warning);
                return;
            }
            //  
            Application.Run(new ProcessViewer());
            //  
            m.ReleaseMutex();
        }

        // 
        ProcessViewer()
        {
            //  
            this.Column = -1;
            //  
            this.SelPos = -1;

            //     
            this.StartPosition = FormStartPosition.CenterScreen;
            // 
            this.Text = "ProcessList";
            // 
            this.Size = new Size(640, 480);     
            //   
            this.Icon = new Icon(GetType(), "Process.ico");

            //     
            this.ProcessPath = new Label();
            this.ProcessPath.Parent = this;
            this.ProcessPath.AutoSize = true;
            this.ProcessPath.Size = new Size(this.ClientSize.Width, 20);
            // 
            this.ProcessPath.Location = new Point(0, this.ClientSize.Height - 50);
            // 
            this.ProcessPath.Anchor = AnchorStyles.Bottom |
                AnchorStyles.Left | AnchorStyles.Right;

            //   
            this.ProcessPathTip = new ToolTip();

            //    
            this.EndProcess = new Button();
            this.EndProcess.Parent = this;
            this.EndProcess.Text = "End Process";
            this.EndProcess.Size = new Size(this.ClientSize.Width, 30);
            this.EndProcess.Location = new Point(0, this.ClientSize.Height - 30);
            this.EndProcess.Anchor = AnchorStyles.Bottom | 
                AnchorStyles.Left | AnchorStyles.Right;
            this.EndProcess.TabIndex = 1;
            //  
            this.EndProcess.Click += new EventHandler(EndProcess_Click);

            //   
            this.ProcessList = new ListView();
            this.ProcessList.Parent = this;
            this.ProcessList.Size = 
                new Size(this.ClientSize.Width, this.ClientSize.Height - 50);
            //  - 
            this.ProcessList.View = View.Details;
            this.ProcessList.Anchor = AnchorStyles.Top | 
                AnchorStyles.Left | AnchorStyles.Right | AnchorStyles.Bottom;
            this.ProcessList.TabIndex = 0;
            //    
            this.ProcessList.HideSelection = false;
            //      
            this.ProcessList.MultiSelect = false;

            //  
            ColumnHeader [] colHeaders = new ColumnHeader[8];
            colHeaders[0] = new ColumnHeader();
            //  
            colHeaders[0].Text = "Process Name";
            colHeaders[0].Width = 90;
            colHeaders[1] = new ColumnHeader();
            //  
            colHeaders[1].Text = "PID";
            colHeaders[1].Width = 40;
            colHeaders[2] = new ColumnHeader();
            //     
            colHeaders[2].Text = "Responding";
            colHeaders[2].Width = 70;
            colHeaders[3] = new ColumnHeader();
            //    
            colHeaders[3].Text = "Main Window Handle";
            colHeaders[3].Width = 115;
            colHeaders[4] = new ColumnHeader();
            //    
            colHeaders[4].Text = "Main Window Title";
            colHeaders[4].Width = 105;
            colHeaders[5] = new ColumnHeader();
            //   
            colHeaders[5].Text = "Handles";
            colHeaders[5].Width = 55;
            colHeaders[6] = new ColumnHeader();
            //    
            colHeaders[6].Text = "Threads";
            colHeaders[6].Width = 55;
            colHeaders[7] = new ColumnHeader();
            //  ,   
            colHeaders[7].Text = "Mem Usage (Kb)";
            colHeaders[7].Width = 95;
            //    
            this.ProcessList.Columns.AddRange(colHeaders);
            
            //     
            this.ProcessList.ColumnClick += new ColumnClickEventHandler(ProcessList_ColumnClick);
            //   
            this.ProcessList.SelectedIndexChanged += new EventHandler(ProcessList_SelectedIndexChanged);

            //   
            this.ProcessList.AllowColumnReorder = true;
            //   
            this.ProcessList.FullRowSelect = true;
            //  
            this.ProcessList.GridLines = true;

            //  
            this.RefreshTimer = new Timer();
            //   
            this.RefreshTimer.Interval = 100;
            //  
            this.RefreshTimer.Tick += new EventHandler(RefreshTimer_Tick);
            // 
            this.RefreshTimer.Start();
        }

        // 
        private void RefreshTimer_Tick(object sender, EventArgs e)
        {
            //  
            this.RefreshTimer.Stop();
            //    
            this.RefreshTimer.Interval = 3000;
            
            //  
            ListViewItem lv;
            //   
            this.ProcessList.BeginUpdate();
            //  
            this.ProcessList.Items.Clear();

            //    
            Process [] proc = Process.GetProcesses();

            for(int i = 0; i < proc.Length; i++)
            {
                try
                {
                    //     
                    lv = this.ProcessList.Items.Add(proc[i].MainModule.ModuleName);
                }
                catch
                {
                    //   
                    continue;
                }

                //     
                lv.SubItems.Add(proc[i].Id.ToString());
                //     
                lv.SubItems.Add((proc[i].Responding) ? "Yes" : "No");
                //    
                if((int)proc[i].MainWindowHandle != 0)
                {
                    //   
                    lv.SubItems.Add(proc[i].MainWindowHandle.ToString());
                    //   
                    lv.SubItems.Add(proc[i].MainWindowTitle);
                    //    (   )
                    if(this.SelPos == -1 && this.Text == proc[i].MainWindowTitle)
                        this.SelPos = i;
                }
                else
                {
                    //   
                    lv.SubItems.Add("-");
                    lv.SubItems.Add("");
                }
                //  
                lv.SubItems.Add(proc[i].HandleCount.ToString());
                //  
                lv.SubItems.Add(proc[i].Threads.Count.ToString());
                //     
                lv.SubItems.Add((proc[i].WorkingSet / 1024).ToString());

                //    
                lv.Tag = i;
            }

            //    
            if(this.Column != -1)
            {
                //  ,   IComparer,
                //       
                this.ProcessList.ListViewItemSorter = new ListViewItemComparer(this.Column);
                // 
                this.ProcessList.Sort();
                //  
                this.ProcessList.ListViewItemSorter = null;
            }    
                    
            //   
            foreach(ListViewItem lvi in this.ProcessList.Items)
            {
                //      ( )
                if((int)lvi.Tag == this.SelPos)
                {
                    //      
                    this.ProcessPath.Text = "Path: " + proc[this.SelPos].MainModule.FileName;
                    //     
                    this.ProcessPathTip.SetToolTip(this.ProcessPath, this.ProcessPath.Text);
                    //  
                    this.ProcessList.Items[lvi.Index].Selected = true; 
                    // ,    
                    this.ProcessList.EnsureVisible(lvi.Index);
                    //    
                    lvi.Focused = true;
                    
                    break;
                }
            }          

            //   
            this.ProcessList.EndUpdate();
            //  
            this.RefreshTimer.Start();
        }

        //    
        private void ProcessList_ColumnClick(object sender, ColumnClickEventArgs e)
        {
            //  
            this.Column = e.Column;
            //  ,   IComparer,
            //       
            this.ProcessList.ListViewItemSorter = new ListViewItemComparer(e.Column);
            // 
            this.ProcessList.Sort();
            //  
            this.ProcessList.ListViewItemSorter = null;
        }

        //  
        private void EndProcess_Click(object sender, EventArgs e)
        {
            //   
            if(this.ProcessList.SelectedIndices.Count == 0)
                return;
            //   
            int item = this.ProcessList.SelectedIndices[0];
            //   
            int id = Convert.ToInt32(this.ProcessList.Items[item].SubItems[1].Text);
            //    
            Process pr = Process.GetProcessById(id);
            
            try
            {
                //     (  )
                if(pr.CloseMainWindow() != true)              
                    // "  -"
                    pr.Kill();    
                //    
                this.ProcessList.Items[item].Remove();

                //  
                this.ProcessPath.Text = "";
                //  
                this.ProcessPathTip.SetToolTip(this.ProcessPath, "");
            }
            catch(Exception ex)
            {
                //  
                MessageBox.Show(ex.Message);
            }
        }

        //  
        private void ProcessList_SelectedIndexChanged(object sender, EventArgs e)
        {
            //   
            if(this.ProcessList.SelectedIndices.Count == 0)
                return;

            //    
            this.SelPos = (int)this.ProcessList.SelectedItems[0].Tag;
            //   
            string pid = this.ProcessList.SelectedItems[0].SubItems[1].Text;
            //    
            Process pr = Process.GetProcessById(Convert.ToInt32(pid));
            //  
            this.ProcessPath.Text = "Path: " + pr.MainModule.FileName;
            //  
            this.ProcessPathTip.SetToolTip(this.ProcessPath, this.ProcessPath.Text);
        }
    }

    // ,      ,
    //      
    class ListViewItemComparer : IComparer 
    {
        //  
        private int col;
        public ListViewItemComparer() 
        {
            col=0;
        }
        public ListViewItemComparer(int column) 
        {
            col=column;
        }
        //  
        public int Compare(object x, object y) 
        {
            //       
            string s1 = ((ListViewItem)x).SubItems[col].Text;
            string s2 = ((ListViewItem)y).SubItems[col].Text;

            int i, j;

            try
            {
                //      
                i = Convert.ToInt32(s1);
                j = Convert.ToInt32(s2);
            }
            catch
            {
                //  
                return String.Compare(s1, s2);
            }

            //  
            return i - j;
        }
    }
}